#version 330 core

uniform sampler2D colorTexture;
uniform sampler2D projectedTexture;
uniform sampler2DShadow shadowMap;

in vec3 worldNormal;
in vec2 tex_uv;
in vec3 cameraNormal; 
in vec3 cameraLightDirection; 
in vec4 shadowCoord;
in vec4 projTexCoord;
in vec4 clipSpacePos;
in vec4 prevClipSpacePos;
in vec3 cameraLightDirectionSpot; //spotlightsource->this fragment
in vec3 spotDir; //spotlightsource->floor

layout(location = 0) out vec4 fragColor;
layout(location = 1) out vec2 motionVector;

vec2 poissonDisk[4] = vec2[](
  vec2( -0.94201624, -0.39906216 ),
  vec2( 0.94558609, -0.76890725 ),
  vec2( -0.094184101, -0.92938870 ),
  vec2( 0.34495938, 0.29387760 )
);


void main()
{	
	//------general shading------------------------------
	vec3 n = normalize(cameraNormal);
	vec3 l = normalize(cameraLightDirection);
	float cosTheta = clamp(dot( n,l ), 0,1 );
	vec3 color = texture(colorTexture, tex_uv).rgb;
	color = color * (cosTheta+0.3); 	

	//------projected texture-----------------------------
 	vec4 projColor = vec4(0.0f);
	if(projTexCoord.w <= 0){
		projColor = textureProj(projectedTexture, projTexCoord.xyw);
	}

	//------spotlight-------------------------------------
	vec3 lSpot = normalize(cameraLightDirectionSpot);
	float cosThetaSpot = clamp(dot(n,lSpot),0,1);
	float cos_outer_cone = 0.6;
	float cos_inner_cone = 0.8;
	int isInnerCone = 0;
	int isOuterCone = 0;

	if(cosThetaSpot > 0.0) {	
		float spotEffect = dot(normalize(spotDir), normalize(-cameraLightDirectionSpot));
		if(spotEffect > cos_inner_cone){
			isInnerCone = 1;	
			color += (color * (spotEffect) * (cosThetaSpot))*3;
		}
		else{
			if (spotEffect > cos_outer_cone){
				isOuterCone = 1;
				float falloff = (spotEffect - cos_outer_cone) / (cos_inner_cone - cos_outer_cone);
				color += (color * spotEffect * (cosThetaSpot) * falloff)*4;
			}
		}	
	}

	//------shadow mapping--------------------------------
	float visibility = 1.0;
	float bias = 0.0008;
	vec4 color4 = vec4(color, 1.0f);

	for (int i=0;i<8;i++){
		visibility -= 0.2*(1.0-texture( shadowMap, vec3(shadowCoord.xy + poissonDisk[i%4]/700.0, (shadowCoord.z-bias)/shadowCoord.w)));
	}
	visibility += 1.1f;
	visibility = clamp(visibility, 0,1);
	if(isInnerCone == 1){visibility = 1;}
	if(isOuterCone == 1){visibility += 0.3;}
	visibility = clamp(visibility, 0,1);

	if(projColor != vec4(0.0f)){
		color4 = projColor * (color4 + 0.5) * (visibility-0.5);
	}else{
		color4 = color4 * visibility;
	}
	fragColor = color4;

	//---motion blur (motion vector)-------------------------
	vec3 NDCPos = (clipSpacePos / clipSpacePos.w).xyz;
	vec3 PrevNDCPos = (prevClipSpacePos / prevClipSpacePos.w).xyz;
	motionVector = (NDCPos - PrevNDCPos).xy;
	//motionVector = vec2(0.1f, 0.0f);//test
}